"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _deepClone = require("../utils/deep-clone");

var _query = require("./query");

// TODO: move from compiler directory as used by multiple modules
var PropFiller = /*#__PURE__*/function () {
  function PropFiller(props) {
    (0, _classCallCheck2.default)(this, PropFiller);
    this.setProps(props);
  }

  (0, _createClass2.default)(PropFiller, [{
    key: "setProps",
    value: function setProps(props) {
      this._props = props;
    }
  }, {
    key: "_getTemplateName",
    value: function _getTemplateName(str) {
      // Note: we use a format like {{name}} instead of ${name} as some linters report errors when
      // template strings are found in regular strings.
      var matches = str.match(/^{{([^{}]*)}}$/);

      if (matches) {
        return matches[1];
      }
    }
  }, {
    key: "_getPropFromObj",
    value: function _getPropFromObj(obj, name) {
      var hasProperty = false;
      var value = undefined;

      try {
        var names = name.split('.');
        var nestedObj; // Getter?

        if (obj.get && typeof obj.get === 'function') {
          nestedObj = obj.get(names[0]);
          hasProperty = true;
        } else {
          nestedObj = obj[names[0]]; // We cannot just use `obj.hasOwnProperty()` here as obj may have a null prototype, e.g.
          // `obj = Object.create(null)`

          hasProperty = Object.prototype.hasOwnProperty.call(obj, names[0]);
        }

        if (names.length > 1) {
          names.shift(); // remove 1st name

          value = this._getPropFromObj(nestedObj, names.join('.')).value;
        } else {
          value = nestedObj;
        }
      } catch (err) {// This can occur when there is no nested value in props. We swallow this and return the
        // defaults for hasProperty and value.
      }

      return {
        value: value,
        hasProperty: hasProperty
      };
    }
  }, {
    key: "_getProp",
    value: function _getProp(name) {
      return this._getPropFromObj(this._props, name);
    }
  }, {
    key: "_getPropOrOriginalString",
    value: function _getPropOrOriginalString(name, str, customizer) {
      var _this$_getProp = this._getProp(name),
          value = _this$_getProp.value,
          hasProperty = _this$_getProp.hasProperty; // We need to analyze hasProperty as the property may exist but the value is undefined, e.g.
      // `{foo: undefined }`


      if (hasProperty) {
        return customizer ? customizer(value) : value;
      } else {
        // We leave the original template parameter strings intact if there isn't a match as the template
        // parameter may match at a secondary layer, e.g. the template parameter is not for the MSON
        // object, but in a validator.
        return str;
      }
    }
  }, {
    key: "fillString",
    value: function fillString(obj, customizer) {
      var _this = this;

      // Is obj a string?
      if (typeof obj === 'string') {
        // Is the string just a template string?
        var name = this._getTemplateName(obj);

        if (name !== undefined) {
          // Replace with the raw prop so that numbers are not converted to strings by replace()
          return this._getPropOrOriginalString(name, obj, customizer);
        } else {
          return obj.replace(/{{([^{}]*)}}/g, function (match, name) {
            return _this._getPropOrOriginalString(name, match, customizer);
          });
        }
      } else {
        // We only fill strings so just return the original object TODO: remove this branch? Is it
        // really needed? Shouldn't obj always be a string?
        return customizer ? customizer(obj) : obj;
      }
    }
  }, {
    key: "_fillAllInner",
    value: function _fillAllInner(items, preventQuery, customizer) {
      var _this2 = this;

      var isQuery = false; // Recursively dive into objects and fill any strings

      var obj = (0, _deepClone.deepCloneWith)(items, function (item, name) {
        if (!preventQuery && !isQuery && (0, _query.isOperator)(name)) {
          // We need to check all attribute names as the operator may be nested
          // e.g. `{ x: $add: [1, 2], y: 3 }`
          isQuery = true;
        }

        var performDefaultClone = false;
        var clonedObject = undefined;

        if (item && item.className) {
          // The item is a component, therefore we should not clone it. TODO: is there a better way to
          // determine this?
          clonedObject = item;
        } else if (typeof item === 'string') {
          clonedObject = _this2.fillString(item, customizer);
        } else if (typeof item === 'function') {
          // e.g. JavaScript action
          clonedObject = item;
        } else {
          performDefaultClone = true;
        }

        return {
          performDefaultClone: performDefaultClone,
          clonedObject: clonedObject
        };
      });
      return {
        obj: obj,
        isQuery: isQuery
      };
    }
  }, {
    key: "fillAll",
    value: function fillAll(items) {
      return this._fillAllInner(items).obj;
    }
  }, {
    key: "fill",
    value: function fill(obj, preventQuery, customizer) {
      if (typeof obj === 'string') {
        return this.fillString(obj, customizer);
      } else {
        var filled = this._fillAllInner(obj, preventQuery, customizer);

        if (preventQuery || !filled.isQuery) {
          return filled.obj;
        } else {
          // We choose to execute the Mongo query in this layer instead of BaseComponent._setProperty()
          // as BaseComponent._setProperty() is called far more frequently and we want to avoid the
          // unneeded overhead.
          return (0, _query.executeQuery)(filled.obj);
        }
      }
    }
  }]);
  return PropFiller;
}();

exports.default = PropFiller;