"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.shouldMove = exports.shouldGetAll = exports.updateDoc = exports.createDoc = exports.shouldCRUD = exports.createForm = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _form = _interopRequireDefault(require("../form"));

var _fields = require("../fields");

var _testUtils = _interopRequireDefault(require("../test-utils"));

var _reorder = require("./reorder");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var createForm = function createForm(props) {
  return new _form.default(_objectSpread({
    fields: [new _fields.TextField({
      name: 'firstName'
    }), new _fields.TextField({
      name: 'lastName'
    })]
  }, props));
};

exports.createForm = createForm;

var shouldCRUD = /*#__PURE__*/function () {
  var _ref = (0, _asyncToGenerator2.default)( /*#__PURE__*/_regenerator.default.mark(function _callee(Store, props) {
    var fieldValues, form, store, created, updated, archived, restored;
    return _regenerator.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            fieldValues = {
              firstName: 'Ella',
              lastName: 'Fitzgerald'
            };
            form = createForm({
              value: fieldValues
            });
            store = new Store(props); // Create

            _context.next = 5;
            return store.createDoc({
              form: form
            });

          case 5:
            created = _context.sent;
            expect(created.id).not.toBeFalsy();
            expect(created.archivedAt).toBeNull();
            expect(created.createdAt).not.toBeFalsy();
            expect(created.updatedAt).not.toBeFalsy();
            expect(created.fieldValues).toEqual(fieldValues);
            expect(created.order).toEqual(_reorder.Reorder.DEFAULT_ORDER); // Get

            _context.t0 = expect;
            _context.next = 15;
            return store.getDoc({
              id: created.id
            });

          case 15:
            _context.t1 = _context.sent;
            (0, _context.t0)(_context.t1).toEqual(created);
            _context.t2 = expect;
            _context.next = 20;
            return store.getDoc({
              where: {
                id: 'missingId'
              }
            });

          case 20:
            _context.t3 = _context.sent;
            (0, _context.t2)(_context.t3).toBeNull();
            form.setValues({
              id: created.id,
              firstName: 'F. Scott'
            }); // Make sure timestamps aren't the same

            _context.next = 25;
            return _testUtils.default.sleepToEnsureDifferentTimestamps();

          case 25:
            _context.next = 27;
            return store.updateDoc({
              id: created.id,
              form: form
            });

          case 27:
            updated = _context.sent;
            expect(updated).toEqual({
              id: created.id,
              archivedAt: null,
              createdAt: created.createdAt,
              updatedAt: updated.updatedAt,
              userId: null,
              fieldValues: {
                firstName: 'F. Scott',
                lastName: 'Fitzgerald'
              },
              order: _reorder.Reorder.DEFAULT_ORDER
            });
            expect(updated.updatedAt).not.toEqual(created.updatedAt);
            _context.t4 = expect;
            _context.next = 33;
            return store.getDoc({
              id: created.id
            });

          case 33:
            _context.t5 = _context.sent;
            (0, _context.t4)(_context.t5).toEqual(updated);
            _context.next = 37;
            return _testUtils.default.sleepToEnsureDifferentTimestamps();

          case 37:
            _context.next = 39;
            return store.archiveDoc({
              id: created.id
            });

          case 39:
            archived = _context.sent;
            expect(archived).toEqual(Object.assign({}, updated, {
              archivedAt: archived.archivedAt,
              updatedAt: archived.updatedAt,
              order: _reorder.Reorder.DEFAULT_ORDER
            }));
            expect(archived.archivedAt).not.toBeFalsy();
            expect(archived.updatedAt).not.toEqual(updated.updatedAt); // Make sure timestamps aren't the same

            _context.next = 45;
            return _testUtils.default.sleepToEnsureDifferentTimestamps();

          case 45:
            _context.next = 47;
            return store.restoreDoc({
              id: created.id
            });

          case 47:
            restored = _context.sent;
            expect(restored).toEqual(Object.assign({}, updated, {
              updatedAt: restored.updatedAt,
              order: _reorder.Reorder.DEFAULT_ORDER
            }));
            expect(restored.archivedAt).toBeNull();
            expect(restored.updatedAt).not.toEqual(archived.updatedAt);

          case 51:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function shouldCRUD(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

exports.shouldCRUD = shouldCRUD;

var createDoc = /*#__PURE__*/function () {
  var _ref2 = (0, _asyncToGenerator2.default)( /*#__PURE__*/_regenerator.default.mark(function _callee2(store, fieldValues) {
    var form;
    return _regenerator.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            form = createForm({
              value: fieldValues
            });
            return _context2.abrupt("return", store.createDoc({
              form: form,
              reorder: true
            }));

          case 2:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function createDoc(_x3, _x4) {
    return _ref2.apply(this, arguments);
  };
}();

exports.createDoc = createDoc;

var createDocs = /*#__PURE__*/function () {
  var _ref3 = (0, _asyncToGenerator2.default)( /*#__PURE__*/_regenerator.default.mark(function _callee3(store) {
    var harryValues, harry, hermioneValues, hermione, ronValues, ron;
    return _regenerator.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            harryValues = {
              firstName: 'Harry',
              lastName: 'Potter',
              order: 0
            };
            _context3.next = 3;
            return createDoc(store, harryValues);

          case 3:
            harry = _context3.sent;
            hermioneValues = {
              firstName: 'Hermione',
              lastName: 'Granger',
              order: 1
            };
            _context3.next = 7;
            return createDoc(store, hermioneValues);

          case 7:
            hermione = _context3.sent;
            _context3.next = 10;
            return store.archiveDoc({
              id: hermione.id
            });

          case 10:
            hermione = _context3.sent;
            ronValues = {
              firstName: 'Ron',
              lastName: 'Weasley'
            };
            _context3.next = 14;
            return createDoc(store, ronValues);

          case 14:
            ron = _context3.sent;
            return _context3.abrupt("return", {
              harry: harry,
              hermione: hermione,
              ron: ron
            });

          case 16:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));

  return function createDocs(_x5) {
    return _ref3.apply(this, arguments);
  };
}();

var updateDoc = /*#__PURE__*/function () {
  var _ref4 = (0, _asyncToGenerator2.default)( /*#__PURE__*/_regenerator.default.mark(function _callee4(store, fieldValues) {
    var form;
    return _regenerator.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            form = createForm({
              value: fieldValues
            });
            return _context4.abrupt("return", store.updateDoc({
              form: form,
              reorder: true
            }));

          case 2:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  }));

  return function updateDoc(_x6, _x7) {
    return _ref4.apply(this, arguments);
  };
}();

exports.updateDoc = updateDoc;
var searchDefaults = {
  after: null,
  before: null,
  first: null,
  order: null,
  showArchived: null
};

var shouldGetAll = /*#__PURE__*/function () {
  var _ref5 = (0, _asyncToGenerator2.default)( /*#__PURE__*/_regenerator.default.mark(function _callee5(Store, props) {
    var store, _yield$createDocs, harry, hermione, ron, all;

    return _regenerator.default.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            store = new Store(props);
            _context5.next = 3;
            return createDocs(store);

          case 3:
            _yield$createDocs = _context5.sent;
            harry = _yield$createDocs.harry;
            hermione = _yield$createDocs.hermione;
            ron = _yield$createDocs.ron;
            all = {
              pageInfo: {
                hasNextPage: false
              },
              edges: [{
                node: harry
              }, {
                node: ron
              }, {
                node: hermione
              }]
            }; // Default props

            _context5.t0 = expect;
            _context5.next = 11;
            return store.getAllDocs(_objectSpread(_objectSpread({}, searchDefaults), {}, {
              showArchived: null
            }));

          case 11:
            _context5.t1 = _context5.sent;
            (0, _context5.t0)(_context5.t1).toEqual(all);
            _context5.t2 = expect;
            _context5.next = 16;
            return store.getAllDocs(_objectSpread(_objectSpread({}, searchDefaults), {}, {
              where: {
                $and: [{
                  $or: [{
                    'fieldValues.firstName': {
                      $iLike: 'h%'
                    }
                  }, {
                    'fieldValues.lastName': {
                      $iLike: 'h%'
                    }
                  }]
                }]
              }
            }));

          case 16:
            _context5.t3 = _context5.sent;
            (0, _context5.t2)(_context5.t3).toEqual(Object.assign({}, all, {
              edges: [{
                node: harry
              }, {
                node: hermione
              }]
            }));
            _context5.t4 = expect;
            _context5.next = 21;
            return store.getAllDocs(_objectSpread(_objectSpread({}, searchDefaults), {}, {
              showArchived: true
            }));

          case 21:
            _context5.t5 = _context5.sent;
            (0, _context5.t4)(_context5.t5).toEqual(Object.assign({}, all, {
              edges: [{
                node: hermione
              }]
            }));
            _context5.t6 = expect;
            _context5.next = 26;
            return store.getAllDocs(_objectSpread(_objectSpread({}, searchDefaults), {}, {
              showArchived: false
            }));

          case 26:
            _context5.t7 = _context5.sent;
            (0, _context5.t6)(_context5.t7).toEqual(Object.assign({}, all, {
              edges: [{
                node: harry
              }, {
                node: ron
              }]
            }));
            _context5.t8 = expect;
            _context5.next = 31;
            return store.getAllDocs(_objectSpread(_objectSpread({}, searchDefaults), {}, {
              order: [['fieldValues.firstName', 'DESC']]
            }));

          case 31:
            _context5.t9 = _context5.sent;
            (0, _context5.t8)(_context5.t9).toEqual(Object.assign({}, all, {
              edges: [{
                node: ron
              }, {
                node: hermione
              }, {
                node: harry
              }]
            }));

          case 33:
          case "end":
            return _context5.stop();
        }
      }
    }, _callee5);
  }));

  return function shouldGetAll(_x8, _x9) {
    return _ref5.apply(this, arguments);
  };
}();

exports.shouldGetAll = shouldGetAll;

var expectDocsToEqual = function expectDocsToEqual(docs, items) {
  var edges = [];
  items.forEach(function (item) {
    edges.push({
      node: {
        fieldValues: {
          firstName: item.firstName
        },
        order: item.order
      }
    });
  });
  expect(docs).toMatchObject({
    edges: edges
  });
};

var shouldMove = /*#__PURE__*/function () {
  var _ref6 = (0, _asyncToGenerator2.default)( /*#__PURE__*/_regenerator.default.mark(function _callee6(Store, props) {
    var store, _yield$createDocs2, harry, hermione, ron, all, ginnyValues, ginny;

    return _regenerator.default.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            store = new Store(props);
            _context6.next = 3;
            return createDocs(store);

          case 3:
            _yield$createDocs2 = _context6.sent;
            harry = _yield$createDocs2.harry;
            hermione = _yield$createDocs2.hermione;
            ron = _yield$createDocs2.ron;
            all = {
              pageInfo: {
                hasNextPage: false
              },
              edges: [{
                node: harry
              }, {
                node: ron
              }, {
                node: hermione
              }]
            }; // Initial ordering

            _context6.t0 = expect;
            _context6.next = 11;
            return store.getAllDocs(searchDefaults);

          case 11:
            _context6.t1 = _context6.sent;
            (0, _context6.t0)(_context6.t1).toEqual(all);
            // Create
            ginnyValues = {
              firstName: 'Ginny',
              lastName: 'Weasley',
              order: 1
            };
            _context6.next = 16;
            return createDoc(store, ginnyValues);

          case 16:
            ginny = _context6.sent;
            _context6.t2 = expectDocsToEqual;
            _context6.next = 20;
            return store.getAllDocs(searchDefaults);

          case 20:
            _context6.t3 = _context6.sent;
            _context6.t4 = [{
              firstName: 'Harry',
              order: 0
            }, {
              firstName: 'Ginny',
              order: 1
            }, {
              firstName: 'Ron',
              order: 2
            }, {
              firstName: 'Hermione',
              order: _reorder.Reorder.DEFAULT_ORDER
            }];
            (0, _context6.t2)(_context6.t3, _context6.t4);
            _context6.next = 25;
            return updateDoc(store, Object.assign({}, ron.fieldValues, {
              id: ron.id,
              order: 0
            }));

          case 25:
            ron = _context6.sent;
            _context6.t5 = expectDocsToEqual;
            _context6.next = 29;
            return store.getAllDocs(searchDefaults);

          case 29:
            _context6.t6 = _context6.sent;
            _context6.t7 = [{
              firstName: 'Ron',
              order: 0
            }, {
              firstName: 'Harry',
              order: 1
            }, {
              firstName: 'Ginny',
              order: 2
            }, {
              firstName: 'Hermione',
              order: _reorder.Reorder.DEFAULT_ORDER
            }];
            (0, _context6.t5)(_context6.t6, _context6.t7);
            _context6.next = 34;
            return updateDoc(store, Object.assign({}, harry.fieldValues, {
              id: harry.id,
              order: 2
            }));

          case 34:
            harry = _context6.sent;
            _context6.t8 = expectDocsToEqual;
            _context6.next = 38;
            return store.getAllDocs(searchDefaults);

          case 38:
            _context6.t9 = _context6.sent;
            _context6.t10 = [{
              firstName: 'Ron',
              order: 0
            }, {
              firstName: 'Ginny',
              order: 1
            }, {
              firstName: 'Harry',
              order: 2
            }, {
              firstName: 'Hermione',
              order: _reorder.Reorder.DEFAULT_ORDER
            }];
            (0, _context6.t8)(_context6.t9, _context6.t10);
            _context6.next = 43;
            return store.archiveDoc({
              id: ginny.id,
              reorder: true
            });

          case 43:
            _context6.t11 = expectDocsToEqual;
            _context6.next = 46;
            return store.getAllDocs(searchDefaults);

          case 46:
            _context6.t12 = _context6.sent;
            _context6.t13 = [{
              firstName: 'Ron',
              order: 0
            }, {
              firstName: 'Harry',
              order: 1
            }, {
              firstName: 'Hermione',
              order: _reorder.Reorder.DEFAULT_ORDER
            }, {
              firstName: 'Ginny',
              order: _reorder.Reorder.DEFAULT_ORDER
            }];
            (0, _context6.t11)(_context6.t12, _context6.t13);
            _context6.next = 51;
            return store.restoreDoc({
              id: ginny.id,
              reorder: true
            });

          case 51:
            _context6.t14 = expectDocsToEqual;
            _context6.next = 54;
            return store.getAllDocs(searchDefaults);

          case 54:
            _context6.t15 = _context6.sent;
            _context6.t16 = [{
              firstName: 'Ron',
              order: 0
            }, {
              firstName: 'Harry',
              order: 1
            }, {
              firstName: 'Ginny',
              order: 2
            }, {
              firstName: 'Hermione',
              order: _reorder.Reorder.DEFAULT_ORDER
            }];
            (0, _context6.t14)(_context6.t15, _context6.t16);

          case 57:
          case "end":
            return _context6.stop();
        }
      }
    }, _callee6);
  }));

  return function shouldMove(_x10, _x11) {
    return _ref6.apply(this, arguments);
  };
}();

exports.shouldMove = shouldMove;