"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.cloneDeep = exports.cloneDeepWith = exports.deepClone = exports.deepCloneWith = void 0;

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

// Highlights:
// - deepCloneWith() is more than 2 times faster than Lodash's cloneDeepWith()! See
//   deep-clone.test.js
// - deepClone() allows undefined values to be cloned, e.g.
//     deepClone({ foo: undefined }) = { foo: undefined }
//   but:
//     cloneDeep({ foo: undefined }) = {}
var isClass = function isClass(object) {
  return object.constructor && object.constructor.name !== 'Object';
};

var deepCloneWithInner = function deepCloneWithInner(object, onNode, key, stack) {
  if (stack.indexOf(object) !== -1) {
    // Bail if we find a circular reference
    return object;
  }

  var doClone = true;
  var clonedVal = object;

  if (onNode) {
    var _onNode = onNode(object, key, object, stack),
        performDefaultClone = _onNode.performDefaultClone,
        clonedObject = _onNode.clonedObject;

    doClone = performDefaultClone;

    if (!doClone) {
      clonedVal = clonedObject;
    }
  }

  if (doClone) {
    if (object === null) {
      return null;
    } else if (Array.isArray(object)) {
      stack.push(object);
      clonedVal = object.map(function (value, key) {
        return deepCloneWithInner(value, onNode, key, stack);
      });
      stack.pop();
    } else if ((0, _typeof2.default)(object) === 'object') {
      stack.push(object);

      if (isClass(object)) {
        // Note: this works for both classes and other objects, but it is much slower than `clonedVal = {}`
        clonedVal = Object.assign(Object.create(Object.getPrototypeOf(object)), object);
      } else {
        clonedVal = {};
      }

      for (var _key in object) {
        clonedVal[_key] = deepCloneWithInner(object[_key], onNode, _key, stack);
      }

      stack.pop();
    }
  }

  return clonedVal;
};

var deepCloneWith = function deepCloneWith(object, onNode) {
  return deepCloneWithInner(object, onNode, undefined, []);
};

exports.deepCloneWith = deepCloneWith;

var deepClone = function deepClone(object) {
  return deepCloneWithInner(object, undefined, undefined, []);
}; // Drop-in replacement for Lodash's cloneDeepWith()


exports.deepClone = deepClone;

var cloneDeepWith = function cloneDeepWith(object, customizer) {
  return deepCloneWith(object, function (value, key, object, stack) {
    var clonedObject = customizer(value, key, object, stack);

    if (clonedObject === undefined) {
      return {
        performDefaultClone: true
      };
    } else {
      return {
        performDefaultClone: false,
        clonedObject: clonedObject
      };
    }
  });
}; // Drop-in replacement for Lodash's cloneDeep()


exports.cloneDeepWith = cloneDeepWith;

var cloneDeep = function cloneDeep(object) {
  return deepClone(object);
};

exports.cloneDeep = cloneDeep;